/*
 * File:   main.c
 * Author: jos_p
 *
 * Created on 11 de junio de 2021, 06:26 PM
 */

// Librerias ----------------------------------------------
#include "config.h"
#include <xc.h>
#include <pic16f887.h>
#include <stdio.h>
#include <stdint.h>
#define _XTAL_FREQ 12000000 // Frecuencia
//------------------------------------------------------------

#define alimentar PORTAbits.RA0
#define vuelta    PORTAbits.RA3 
#define motorOn   PORTBbits.RB0 

// Botones para cambio de hora y fecha
#define boton1 PORTAbits.RA2
#define boton2 PORTAbits.RA1

//------------------------------------------------------------

//Conexiones LCD
#define LCD_RS       PORTDbits.RD5
#define LCD_EN       PORTDbits.RD4
#define LCD_D4       PORTDbits.RD0
#define LCD_D5       PORTDbits.RD1
#define LCD_D6       PORTDbits.RD2
#define LCD_D7       PORTDbits.RD3
#define LCD_RS_DIR   TRISDbits.TRISD5
#define LCD_EN_DIR   TRISDbits.TRISD4
#define LCD_D4_DIR   TRISDbits.TRISD0
#define LCD_D5_DIR   TRISDbits.TRISD1
#define LCD_D6_DIR   TRISDbits.TRISD2
#define LCD_D7_DIR   TRISDbits.TRISD3
#include "lcd.h"
//------------------------------------------------------------


// Prototipo de funciones -------------------------------------
void I2C_Init(uint32_t freq);
void I2C_Start(void);
void I2C_Repeated_Start(void);
void I2C_Stop(void);
void I2C_Write(uint8_t datos);
uint8_t I2C_Read(uint8_t ack);
uint8_t BCD_Decimal(uint8_t numero);
uint8_t Decimal_BCD(uint8_t numero);
void RTC_display(); // Muestra la hora y fecha
uint8_t edit(uint8_t x, uint8_t y, uint8_t parametro); // Edita hora y fecha
void blink(); // Parpadeo
__bit debounce(); //antirrebote
//---------------------------------------------------------------


// Variables -----------------------------------------------------
uint8_t i, second, minute, hour, day, month, year;
int cont=0;
int H=0, M=0, hF, mF, buffer = 0;
//;---------------------------------------------------------------

// Funcion Principal -------------------
void main(void){
  ANSELH = 0;         // puertos digitales                
  ANSEL = 0;
 
  TRISBbits.TRISB0 = 0; //Puerto B2 como salida (Motor)
  TRISBbits.TRISB2 = 0;
  TRISA = 0xFF;       //Puerto A como entrada
  
  PORTB = 0X00; // Limpieza de puertos
  PORTA = 0X00;
  
  __delay_ms(1000);
 
  I2C_Init(100000);  // Inicializacion del I2C con una frecuencia de 100kHz
  LCD_Begin();        // inicializa la LCD
  
  
  while(1) {
  
      // Edicion 10/07/2021 ----*---*-*-*-*--*-*-*-*-*-*-*-*-*-*-*--*-*-*-*-*-*
    if(!boton2 && boton1){
            
        LCD_Goto(16, 1);
        LCD_Print("A"); 
        buffer = 1;
        hF = 1;
        M = 0;
        H = 0;
            
        if(hF==1){
            hF = 0;
            mF = 1;
            H = edit(7, 1, H);
        } 
        if(mF == 1){
            hF =0;
            mF = 0;
            M = edit(10,1, M);  
        }
        
        LCD_Goto(16, 1);
        LCD_Print(" ");    
    }
    // Edicion -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
     
    //RTC - Reloj  ************************************************  
    if(!boton1)       // Si se presiona el botn B1
    if(debounce()){   // Llama a la funcin antirrebote
      i = 0;
      hour   = edit(7,  1, hour);
      minute = edit(10, 1, minute);
      day    = edit(7,  2, day);
      month  = edit(10, 2, month);
      year   = edit(15, 2, year);
    
      while(debounce());  // Espera a que se suelte el botn B1)
 
      // Convierte decimal a BCD
      minute = Decimal_BCD(minute);
      hour   = Decimal_BCD(hour);
      day    = Decimal_BCD(day);
      month  = Decimal_BCD(month);
      year   = Decimal_BCD(year);
      // ------------------------
      
      // Escribe datos en RTC DS1307
      I2C_Start();         // start I2C
      I2C_Write(0xD0);     // RTC chip address
      I2C_Write(0);        // send register address
      I2C_Write(0);        // reset seconds and start oscillator
      I2C_Write(minute);   // write minute value to RTC chip
      I2C_Write(hour);     // write hour value to RTC chip
      I2C_Write(1);        // write day value (not used)
      I2C_Write(day);      // write date value to RTC chip
      I2C_Write(month);    // write month value to RTC chip
      I2C_Write(year);     // write year value to RTC chip
      I2C_Stop();          // stop I2C
 
      __delay_ms(200);
 
    }
 
    // Lee la hora y la fecha actuales del RTC
    I2C_Start();           // start I2C
    I2C_Write(0xD0);       // RTC chip address
    I2C_Write(0);          // send register address
    I2C_Repeated_Start();  // restart I2C
    I2C_Write(0xD1);       // initialize data read
    second = I2C_Read(1);  // read seconds from register 0
    minute = I2C_Read(1);  // read minutes from register 1
    hour   = I2C_Read(1);  // read hour from register 2
    I2C_Read(1);           // read day from register 3 (not used)
    day  = I2C_Read(1);    // read date from register 4
    month  = I2C_Read(1);  // read month from register 5
    year   = I2C_Read(0);  // read year from register 6
    I2C_Stop();            // stop I2C
    
    RTC_display();    // Muestra la hora y fecha
 
    __delay_ms(50);    
  }
  
  return;
}

//Funciones---------------------------------------------------------------------
void I2C_Init(uint32_t freq){
    SSPCON = 0B00101000;
    SSPADD  = (_XTAL_FREQ/(4*freq)) - 1;  // 
    SSPSTAT = 0;
}

void I2C_Start(){
    while((SSPSTAT & 0X04)||(SSPCON2 & 0X1F));
    SSPCON2bits.SEN = 1;
}

void I2C_Repeated_Start(void){
    while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F));  //Espera a que el mdulo MSSP est libre (no ocupado)
    SSPCON2bits.RSEN = 1;  // Inicia una condicin de inicio repetida
}

void I2C_Stop(void){
  while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F));  
  SSPCON2bits.PEN = 1;  // Inicia condicion de parada
}
  
void I2C_Write(uint8_t datos){
  while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F)); 
  SSPBUF = datos;  // Actualiza el buffer
}

uint8_t I2C_Read(uint8_t ack){
  uint8_t _datos;
  while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F));  
  SSPCON2bits.RCEN = 1;
  while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F)); 
  _datos = SSPBUF;  // Lee datos del bfer
  while ((SSPSTAT & 0x04) || (SSPCON2 & 0x1F));  
  // Envia el acknowledge? (depende del ack. Si es 1 envia, de lo controraio no envia) 
  SSPCON2bits.ACKDT = (ack)?0:1;
  SSPCON2bits.ACKEN = 1;
  return _datos;  // Retorna los datos leidos
}

// Antirebote
__bit debounce(){
  uint8_t count = 0;
  for(uint8_t i = 0; i < 5; i++) {
    if (boton1 == 0)
      count++;
    __delay_ms(10);
  }
  if(count > 2)  return 1;
  else           return 0;
}

uint8_t BCD_Decimal(uint8_t number) {
  return((number >> 4) * 10 + (number & 0x0F));
}

uint8_t Decimal_BCD(uint8_t number) {
  return(((number / 10) << 4) + (number % 10));
}

void RTC_display(){
  
  static char Time[] = "HORA: 00:00:00  ";
  static char Date[] = "FECHA:00/00/2020";
 
  // Convierte BCD a decimal ---------------------------------------------------
  second = BCD_Decimal(second);
  minute = BCD_Decimal(minute);
  hour   = BCD_Decimal(hour);
  day    = BCD_Decimal(day);
  month  = BCD_Decimal(month);
  year   = BCD_Decimal(year);
  // ---------------------------------------------------------------------------

    if ((hour == H && minute == M)){
      
        if (cont == 0 && vuelta == 1){
                motorOn = 1; 
                __delay_ms(2000);
        } 
        else if (vuelta == 0) {
           
            if(cont <10 ){
                motorOn = 1;
                cont++;
                __delay_ms(1000);
            } 
            else {
                motorOn= 0;
        }
    } 
    // Configuracion sensor IR -----------------------------
    } 
    else if(!alimentar){
        PORTBbits.RB0 = 1;
        motorOn = 1;
        __delay_ms(100);
    }
    else{
        motorOn = 0;
        PORTBbits.RB0 = 0;
    }
     
    if (hour != H || minute != M){
       cont =0;
    }
  
  // Actualiza hora y fecha ----------------------------------------
  Time[6]  = hour   / 10 + '0';
  Time[7]  = hour   % 10 + '0';
  Time[9]  = minute / 10 + '0';
  Time[10] = minute % 10 + '0';
  Time[12] = second / 10 + '0';
  Time[13] = second % 10 + '0';
  
  Date[6]  = day  / 10 + '0';
  Date[7]  = day  % 10 + '0';
  Date[9]  = month  / 10 + '0';
  Date[10] = month  % 10 + '0';
  Date[14] = year   / 10 + '0';
  Date[15] = year   % 10 + '0';
 
  LCD_Goto(1, 1);    // Columna 1, Fila 1
  LCD_Print(Time);   // Escribe Hora
  LCD_Goto(1, 2);    // Columna 1, Fila 2
  LCD_Print(Date);   // Escribe Fecha
  
}

void blink(){
  uint8_t j = 0;
  while(j < 100 && boton1 && boton2) {
    j++;
    __delay_ms(5);
  }
}

uint8_t edit(uint8_t x, uint8_t y, uint8_t parametro){
  while(debounce());  // Llama a la funcin antirrebote (esperar a que se libere B1)
 
  while(1) {
 
    while(!boton2){    // Si boton2 es presionado
   
      parametro++;
      if(i == 0 && parametro > 23)   // Si hours > 23 entonces hours = 0
        parametro = 0;
      if(i == 1 && parametro > 59)   // Si minutes > 59 entonces minutes = 0
        parametro = 0;
      if(i == 2 && parametro > 31)   // Si date > 31 entonces date = 1
        parametro = 1;
      if(i == 3 && parametro > 12)   // Si month > 12 entonces month = 1
        parametro = 1;
      if(i == 4 && parametro > 30)   // Si year > 30 entonces year = 20
        parametro = 20;
    
      if(hF == 0 && mF == 1 && parametro > 23){
          parametro = 0;
      }
      if(mF == 0 && parametro > 59){
          parametro = 0;
      }
 
      LCD_Goto(x, y);
      LCD_PutC(parametro / 10 + '0');
      LCD_PutC(parametro % 10 + '0');
      __delay_ms(200);
 
    }
 
    LCD_Goto(x, y);
    LCD_Print("  ");  // Escribe dos espacios
    blink();
 
    LCD_Goto(x, y);
    LCD_PutC(parametro / 10 + '0');
    LCD_PutC(parametro % 10 + '0');
    blink();
 
    if(!boton1)           // Si se presiona el botn B1
    if(debounce()){       // Llama a la funcin antirrebote
      i++;                // Incrementar 'i' para el siguiente parmetro
      return parametro;   // Devuelve el valor del parmetro
    }
 
  }
 
}